package ascensore.persistence;

import ascensore.model.*;

import java.io.Reader;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

public class MyEdificiReader implements EdificiReader {

	@Override
	public List<Edificio> readAll(Reader reader) throws BadFileFormatException, IOException {

		List<Edificio> edifici = new ArrayList<>();
		String line = null;

		/*
		 * Hotel Miralago, -2, 4, multipiano Condominio "I Girasoli", -1, 6,
		 * Singolo Casa Zannoni, 0, 2, mULtipiano
		 */

		try {
			BufferedReader myReader = new BufferedReader(reader);
			while ((line = myReader.readLine()) != null) {
				StringTokenizer tokenizer = new StringTokenizer(line, ",\n\r");
				String nome = tokenizer.nextToken().trim();
				checkNotEmpty(nome, "nome");
				String pMin = tokenizer.nextToken().trim();
				checkNotEmpty(nome, "pianoMin");
				String pMax = tokenizer.nextToken().trim();
				checkNotEmpty(nome, "pianoMax");
				String tipo = tokenizer.nextToken().toUpperCase().trim();
				checkNotEmpty(nome, "modo");

				int pianoMin = Integer.parseInt(pMin);
				int pianoMax = Integer.parseInt(pMax);
				if (pianoMax <= pianoMin)
					throw new BadFileFormatException(
							"PianoMax dev'essere > PianoMin");

				Modo modo = Modo.valueOf(tipo);
				edifici.add(new Edificio(nome, pianoMin, pianoMax, 
						Ascensore.of(modo, pianoMin, pianoMax)));
			}
			return edifici;
		} catch (NoSuchElementException | IllegalArgumentException e) {
			throw new BadFileFormatException(e);
		}
	}

	private void checkNotEmpty(String nome, String tokenName) throws BadFileFormatException {
		if (nome.isEmpty())
			throw new BadFileFormatException(tokenName);		
	}

	public static void main(String[] args) throws BadFileFormatException,
			IOException {
		EdificiReader reader = new MyEdificiReader();
		List<Edificio> elencoEdifici = reader.readAll(new FileReader(
				"Edifici.txt"));
		for (Edificio t : elencoEdifici)
			System.out.println(t);
	}

}
