package ascensore.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Color;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import ascensore.model.*;
import ascensore.ui.pulsantiera.Pulsantiera;

public class MainPanel extends JPanel implements MainView, ActionListener {

	private static final long serialVersionUID = 1L;

	private Controller controller;
	private JComboBox<Edificio> edificiCombo;
	private JTextField displayPiano;
	private JTextArea txtarea;
	private Pulsantiera pulsantiera;

	public MainPanel(Controller controller) {
		this.controller = controller;
		this.controller.setMainView(this);
		
		initGUI();
				
		if (edificiCombo.getItemCount() > 0) {
			edificiCombo.setSelectedIndex(0);
		}
	}

	private void initGUI() {
		setLayout(new BorderLayout());
		Box topLine = new Box(BoxLayout.LINE_AXIS);
		edificiCombo = new JComboBox<>(controller.getElencoEdifici().toArray(new Edificio[0]));
		edificiCombo.addActionListener(this);
		topLine.add(edificiCombo);
		displayPiano = new JTextField("0", 1);
		displayPiano.setFont(new Font("Verdana", Font.BOLD, 40));
		displayPiano.setAlignmentX(CENTER_ALIGNMENT);
		displayPiano.setForeground(Color.GREEN);
		displayPiano.setBackground(Color.BLACK);
		topLine.add(displayPiano);
		this.add(topLine, BorderLayout.NORTH);
		txtarea = new JTextArea(30, 60);
		txtarea.setFont(new Font("Courier New", Font.PLAIN, 11));
		JScrollPane scrollPane = new JScrollPane(txtarea);
		this.add(scrollPane, BorderLayout.EAST);
		pulsantiera = new Pulsantiera();
		pulsantiera.setLayout(new BoxLayout(pulsantiera, BoxLayout.PAGE_AXIS));
		pulsantiera.setBackground(Color.YELLOW);
		pulsantiera.setPreferredSize(new Dimension(100, 60));
		pulsantiera.addActionListener(this);
		this.add(pulsantiera, BorderLayout.CENTER);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == edificiCombo) {
			Ascensore asc = null;
			Edificio edificio = (Edificio) edificiCombo.getSelectedItem();
			asc = edificio.getAscensore();
			controller.setAscensore(asc);
			pulsantiera.setAscensore(asc);
		} else if (e.getSource() == pulsantiera) {
			pulsantePremuto(e.getActionCommand());
		}
	}

	private void pulsantePremuto(String actionCommand) {
		int piano = Integer.parseInt(actionCommand);
		controller.vaiAlPiano(piano);
	}
	
	@Override
	public void appendStatus(String status) {
		txtarea.append(status);
	}

	@Override
	public void clearStatus() {
		txtarea.setText("");
	}

	@Override
	public void setPianoCorrente(int pianoCorrente) {
		displayPiano.setText("" + pianoCorrente);
	}

}
