package ascensore.ui.pulsantiera;

import java.awt.*;
import java.awt.geom.*;

import javax.swing.*;
import javax.swing.border.BevelBorder;

/*
 * Code originally published at http://www.coderanch.com/t/339222/GUI/java/Button-Border 
 * */
public class RoundButton extends JButton {

	private static final long serialVersionUID = 1L;

	public RoundButton(String label) {
		super(label);
		// These statements enlarge the button so that it
		// becomes a circle rather than an oval.
		Dimension size = getPreferredSize();
		size.width = size.height = Math.max(size.width, size.height);
		setPreferredSize(size);
		// ED ADDED to force circle also in non-circle containers
		setMinimumSize(size); setMaximumSize(size); //ED 
		// This call causes the JButton not to paint the background.
		// This allows us to paint a round background.
		setContentAreaFilled(false);
	}

	// Paint the round background and label.
	protected void paintComponent(Graphics g) {
		if (getModel().isArmed()) {
			// You might want to make the highlight color
			// a property of the RoundButton class.
			g.setColor(Color.lightGray);
		} else {
			g.setColor(getBackground());
		}

		g.fillOval(0, 0, getSize().width - 1, getSize().height - 1);

		// This call will paint the label and the focus rectangle.
		super.paintComponent(g);
	}

	// Paint the border of the button using a simple stroke.
	protected void paintBorder(Graphics g) {
		g.drawOval(0, 0, getSize().width - 1, getSize().height - 1);
		setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
	}

	// Hit detection.
	Shape shape;

	public boolean contains(int x, int y) {
		// If the button has changed size, make a new shape object.
		if (shape == null || !shape.getBounds().equals(getBounds())) {
			shape = new Ellipse2D.Float(0, 0, getWidth(), getHeight());
		}
		return shape.contains(x, y);
	}


}